package MovaMotion;

use strict;

use base qw( MT::App );

use MT;
use MT::Blog;
use MT::Session;
use MT::Author;
use MT::Entry;
use MT::Tag;
use File::Spec;
use MT::Util qw( perl_sha1_digest_hex );

@MovaMotion::ISA = qw( MT::App );

sub init_request {
    my $app = shift;
    $app->SUPER::init_request( @_ );
    $app->{ default_mode } = 'view';
    $app->add_methods( view => \&_view );
    $app->add_methods( login => \&_login );
    $app->add_methods( logout => \&_logout );
    $app->add_methods( redirect => \&_redirect );
    my $ua = $app->get_header( 'User-Agent' );
    $app->{ requires_login } = 1 if ( $ua =~ /Mozilla/ );
    $app;
}

sub _view {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    my %param; my $tmpl;
    my $plugin = MT->component( 'MovaMotion' );
    $app->config( 'TemplatePath', File::Spec->catdir( $plugin->path, 'tmpl' ) );
    my $blog_id  = $app->param( 'blog_id' );
    my $offset = $app->param( 'offset' ) || 0;
    my $limit  = $app->param( 'limit' ) || 20;
    my $tag = lc ( $app->param( 'reply' ) );
    my $reply2 = $app->param( 'reply2' );
    my $author = $app->param( 'author' );
    my $mailpost = $app->param( 'mailpost' );
    $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
    $offset = 0 if ( $offset !~ /^[0-9]{1,}$/ );
    $limit = 20 if ( $limit !~ /^[0-9]{1,}$/ );
    my $blog = $app->blog;
    %param->{ blog_id } = $blog_id;
    %param->{ offset } = $offset;
    %param->{ limit } = $limit;
    %param->{ mailpost } = $mailpost;
    my $sessid = $app->param( 'sessid' );
    my $type = $app->param( 'type' );
    my $user;
    if ( $ua =~ /Mozilla/ ) {
        $user = $app->user;
        $sessid = '';
    } else {
        $user = _get_auther( $app, $sessid );
    }
    if ( (! defined $user ) || (! $blog_id ) ) {
        $tmpl = 'login.tmpl';
        return $app->build_page( $tmpl, \%param );
    } else {
        unless ( _can_access ( $user, $blog_id ) ) {
            return $plugin->translate( 'Permission denied' );
        }
        %param->{ username } = $user->name;
        if ( $type eq 'set_uid' ) {
            my $return_url = $app->param( 'return_url' );
            $return_url =~ s/set_uid=[a-z]{1,}//;
            my $mobile_id = _get_mobile_id( $app );
            if ( $mobile_id ) {
                $user->mobile_id( $mobile_id );
                $user->save or return $user->errstr;
                $return_url .= ';set_uid=successful';
            } else {
                $return_url .= ';set_uid=failed';
            }
            $app->redirect( $return_url );
        }
        if ( _can_post ( $user, $blog_id ) ) {
            if ( $type eq 'createmail' ) {
                %param->{ createmail } = 1;
                my $postaddress = $plugin->get_config_value( 'movamotion_address', 'blog:'. $blog_id );
                my $token = $user->mail_token;
                $postaddress =~ s/@/+$token@/;
                %param->{ postaddress } = $postaddress;
            }
            if ( $type eq 'mailpost' ) {
                my $res = MT::Plugin::MovaMotion::_post_from_email( $blog, $user->id );
                if ( $res ) {
                    my $return_url = $app->base . $app->uri( mode => 'view',
                                                             args => { blog_id => $blog_id,
                                                                       sessid => $sessid,
                                                                       do_post => 1, } );
                    return $app->redirect( $return_url );
                }
            }
            if ( $type eq 'entry' ) {
                %param->{ entry } = 1;
            }
            if ( $type eq 'post' ) {
                my $title = $app->param( 'title' );
                if ( $title ) {
                    my $entry = MT::Entry->new;
                    $entry->blog_id( $blog_id );
                    $entry->status( MT::Entry::RELEASE() );
                    $entry->author_id( $user->id );
                    $entry->title( $title );
                    $entry->text( $app->param( 'text' ) );
                    $entry->text_more( $app->param( 'text_more' ) );
                    $entry->excerpt( $app->param( 'excerpt' ) );
                    $entry->keywords( $app->param( 'keywords' ) );
                    my @tags;
                    for my $reply ( $title =~ m/@([a-zA-Z0-9]{1,})/g ) {
                        push ( @tags, lc ( $reply ) );
                    }
                    $entry->set_tags ( @tags ) if ( @tags );
                    $entry->save
                        or die $entry->errstr;
                    if ( _can_rebuild ( $user, $blog_id ) ) {
                        MT::Util::start_background_task(
                            sub { $app->rebuild_entry( Entry => $entry->id,
                                                       BuildDependencies => 1 );
                            }
                        );
                    }
                    my $return_url = $app->base . $app->uri( mode => 'view',
                                                             args => { blog_id => $blog_id, sessid => $sessid } );
                    return $app->redirect( $return_url );
                }
            }
            %param->{ can_post } = 1;
        }
    }
    my $last = $offset + $limit;
    my $terms = { blog_id => $blog_id, status => MT::Entry::RELEASE() };
    my $count;
    if ( $tag ) {
        my $obj = MT::Tag->load( { name => $tag } );
        $count = MT::Entry->tagged_count( $obj->id, $terms ) if defined $obj;
    } else {
        if ( $author ) {
            my $obj = MT::Author->load( { name => $author } );
            if ( defined $obj ) {
                $terms->{ author_id } = $obj->id;
            } else {
                $author = undef;
            }
        }
        $count = MT::Entry->count( $terms );
    }
    if ( $last < $count ) {
        %param->{ next } = 1;
        %param->{ nextoffset } = $last;
    }
    if ( $offset ) {
        %param->{ prev } = 1;
        my $prev = $last - $limit * 2;
        $prev = 0 if ( $prev < 0 );
        %param->{ prevoffset } = $prev;
    }
    my $return_url = $app->base . $app->uri . '?' . $app->query_string;
    %param->{ return_url } = $return_url;
    %param->{ sessid } = $sessid;
    my $set_uid = $app->param( 'set_uid' );
    if ( $set_uid ) {
        %param->{ status_message } = 1;
        %param->{ set_uid } = $set_uid;
    }
    %param->{ tag } = $tag;
    %param->{ author } = $author;
    %param->{ reply2 } = $reply2;
    if ( $type eq 'settings' ) {
        %param->{ settings } = 1;
    }
    $tmpl = 'view.tmpl';
    return $app->build_page( $tmpl, \%param );
}

sub _login {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    my $blog_id  = $app->param( 'blog_id' );
    if ( $ua =~ /Mozilla/ ) {
        if ( $app->user ) {
            my $return_url = $app->base . $app->uri( mode => 'view',
                                                     args => { blog_id => $blog_id } );
            return $app->redirect( $return_url );
        }
    }
    my $plugin = MT->component( 'MovaMotion' );
    my $type = $app->param( 'type' );
    my $author;
    if ( $type eq 'easy' ) {
        my $mobile_id = _get_mobile_id( $app );
        my @authors = MT::Author->search_by_meta( mobile_id => $mobile_id );
        return $plugin->translate( 'Unknown user' ) unless @authors;
        $author = $authors[0];
    } else {
        my $username = $app->param( 'username' );
        my $password = $app->param( 'password' );
        $author = MT::Author->load( { name => $username } );
        return $plugin->translate( 'Unknown user' ) unless defined $author;
        return $plugin->translate( 'Invalid password' ) unless $author->is_valid_password( $password );
    }
    my $sess = MT::Session->get_by_key ( { email => $author->email,
                                           name  => $author->name,
                                           kind  => 'SI' } );
    $sess->start( time );
    unless ( $sess->id ) {
        $sess->id ( $app->make_magic_token );
    }
    $sess->save or return 'Error saving session';
    my $return_url = $app->base . $app->uri( mode => 'view',
                                             args => { blog_id => $blog_id, sessid => $sess->id } );
    return $app->redirect( $return_url );
}

sub _logout {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( $ua =~ /Mozilla/ ) {
        $app->logout();
    }
    my $sessid = $app->param( 'sessid' );
    my $sess = MT::Session->load ( { id => $sessid,
                                     kind  => 'SI' } );
    if ( defined $sess ) {
        $sess->remove or die $sess->errstr;
    }
    my $blog_id  = $app->param( 'blog_id' );
    $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
    my $return_url = $app->base . $app->uri( mode => 'view',
                                             args => { blog_id => $blog_id, logout => 1 } );
    return $app->redirect( $return_url );
}

sub _redirect {
    my $app = shift;
    my $plugin = MT->component( 'MovaMotion' );
    $app->config( 'TemplatePath', File::Spec->catdir( $plugin->path, 'tmpl' ) );
    my $url = $app->param( 'url' );
    my %param; my $tmpl;
    %param->{ url } = $url;
    $tmpl = 'redirect.tmpl';
    return $app->build_page( $tmpl, \%param );
}

sub _get_auther {
    my ( $app, $sessid ) = @_;
    my $author;
    my $sess = MT::Session->load ( { id => $sessid,
                                     kind  => 'SI' } );
    if ( defined $sess ) {
       my $sess_timeout = $app->config->UserSessionTimeout;
       if ( ( time - $sess->start ) < $sess_timeout ) {
           $author = MT::Author->load( { name => $sess->name } );
           $sess->start( time );
           $sess->save or die $sess->errstr;
       } else {
            my $blog_id  = $app->param( 'blog_id' );
            $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
            my $return_url = $app->base . $app->uri( mode => 'view', args => { blog_id => $blog_id, logout => 1 } );
            $app->redirect( $return_url );
       }
    }
    return $author;
}

sub _can_post {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_post = $user->is_superuser;
    unless ( $can_post ) {
        $can_post = $user->permissions( $blog_id )->can_post;
    }
    return $can_post;
}

sub _can_rebuild {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_rebuild = $user->is_superuser;
    unless ( $can_rebuild ) {
        $can_rebuild = $user->permissions( $blog_id )->can_rebuild;
    }
    return $can_rebuild;
}

sub _can_access {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_access = $user->is_superuser;
    unless ( $can_access ) {
        $can_access = MT::Permission->load( { blog_id => $blog_id, author_id => $user } );
    }
    return $can_access;
}

sub _get_mobile_id {
    my $app = shift;
    my $mobile_id;
    my $user_agent = $app->get_header( 'User-Agent' );
    my @browswer = split( m!/!, $user_agent );
    if ( $browswer[0] eq 'DoCoMo' ) {
        if ( $user_agent =~ /^.*(ser[0-9]{11,}).*$/ ) {
            $mobile_id = $1;
        }
    } else {
        my $x_up_subno = $user_agent = $app->get_header( 'X_UP_SUBNO' );
        # AU
        if ( $x_up_subno ) {
            $mobile_id = $x_up_subno;
        } else {
             # Softbank
             my $x_jphone_uid = $app->get_header( 'X_JPHONE_UID' );
             if ( $x_jphone_uid ) {
                 $mobile_id = $x_jphone_uid;
             }
        }
    }
    if ( $mobile_id ) {
        if ( eval { require Digest::MD5; 1; } ) {
            $mobile_id = Digest::MD5::md5_hex( $mobile_id );
        } else {
            $mobile_id = perl_sha1_digest_hex( $mobile_id );
        }
        return $mobile_id if $mobile_id;
    }
    return '';
}

1;